#ifndef MEMORYH
#define MEMORYH


/* Error codes */

#define MEMORY_ROM_ERROR_SIZE           0
#define MEMORY_ROM_ERROR_AMIROM_VERSION 1
#define MEMORY_ROM_ERROR_AMIROM_READ    2
#define MEMORY_ROM_ERROR_KEYFILE        3
#define MEMORY_ROM_ERROR_EXISTS_NOT     4
#define MEMORY_ROM_ERROR_FILE           5
#define MEMORY_ROM_ERROR_KICKDISK_NOT   6
#define MEMORY_ROM_ERROR_CHECKSUM       7
#define MEMORY_ROM_ERROR_KICKDISK_SUPER 8
#define MEMORY_ROM_ERROR_BAD_BANK       9


extern ULO ypos, xpos;

extern UBY emem[];
extern void memoryEmemCardAdd(ememinitfunc cardinit, ememmapfunc cardmap);
extern void memoryEmemCardInit(void);
extern void memoryEmemSet(ULO index, ULO data);

extern UBY dmem[];

extern void memorystart(void);  /* Code alignment label */
extern ULO wriorgadr;

extern ULO sprite_delay;

extern UBY *address_to_ptr(ULO address);
extern ULO config_memory_chipsize;
extern ULO config_memory_fastsize;
extern ULO config_memory_bogosize;
extern BOOLE config_memory_nokick;
extern BOOLE config_memory_space_32bit;
extern ULO config_memory_kicksize;
extern STR config_memory_kickname[256];
extern STR config_memory_kick2name[256];
extern STR config_memory_kicknameloaded[256];
extern ULO config_memory_romversion;
extern STR kickstart_idstring[80];
extern STR *memoryRomIdentify(STR *s);
extern STR config_memory_keyname[];
extern ULO config_memory_kickbase;

extern ULO screenptr;

extern ULO fetb(ULO adress);
extern ULO fetw(ULO adress);
extern ULO fetl(ULO adress);

extern void wrib(ULO data,ULO adress);
extern void wriw(ULO data,ULO adress);
extern void wril(ULO data,ULO adress);
extern void dumpioblock(void);
extern void memoryInit(void);
extern void memoryShutdown(void);
extern void hexdump(ULO);
extern void memoryColorTranslationInit(void);
extern ULO dmaconr,dmacon;
extern ULO intenar,intena,intreq;
extern ULO potgor;


/* Screen related registers */

extern ULO lof,diwxleft,diwxright,diwytop,diwybottom,ddfstrt,ddfstrtl,
    ddfstop,ddfstopl,topbottomflag,bpl1pt,bpl2pt,bpl3pt,bpl4pt,bpl5pt,bpl6pt,
    bplcon0,bplcon1,bplcon2,bpl1mod,bpl2mod,diwstrt,diwstop;

extern ULO DDFstartpos,DDFnumberofwords,DIWfirstvisiblepos,DIWlastvisiblepos;
extern ULO pagenr;

extern ULO adcon;

extern ULO dskpt,dsklen,dsksync;

/* Copper registers */

extern ULO copcon,cop1lc,cop2lc;

/* Blitter registers */

extern ULO linenum,linecount,linelength,blitterdmawaiting;

extern UBY cmem[];
extern UBY *fmem;
extern UBY bmem[];
extern UBY kmem[];


extern ULO sprpt[8],sprx[8],spry[8],sprly[8],spratt[8];
extern ULO spritestate[8];
extern ULO sprite16col[8];
extern UWO sprdat[8][2];

extern ULO oddscroll,evenscroll;
extern ULO oddhiscroll,evenhiscroll;

extern void dsetb(UBY data);
extern void dsetw(UWO data);
extern void dsetl(ULO data);
extern void dmem_setcounter(ULO val);
extern ULO dmem_getcounter(void);
extern void dmem_clear(void);

extern void memoryBankSet(readfunc rb, readfunc rw, readfunc rl,
			  writefunc wb, writefunc ww, writefunc wl,
			  UBY *basep, ULO bank, ULO basebank,
			  BOOLE datadirect);


extern regreadfunc ioread[256];
extern regwritefunc iowrite[256];

extern void wintreq(ULO,ULO);

/*===============*/
/* Memory tables */
/*===============*/

extern readfunc memory_bank_readbyte[65536];
extern readfunc memory_bank_readword[65536];
extern readfunc memory_bank_readlong[65536];
extern writefunc memory_bank_writebyte[65536];
extern writefunc memory_bank_writeword[65536];
extern writefunc memory_bank_writelong[65536];
extern UBY *memory_bank_pointer[65536];
extern UBY *memory_bank_datapointer[65536];





#endif
