#ifndef SOUNDH
#define SOUNDH


/*==========================*/
/* Sound device information */
/*==========================*/

typedef void (*soundDevSampleAddFunc)(ULO);
typedef ULO (*soundDevModeSetFunc)(void);
typedef void (*soundDevEmulationStartingFunc)(void);
typedef void (*soundDevEmulationStoppingFunc)(void);
typedef void (*soundDevBufferPlayFunc)(void);

typedef struct {
  BOOLE found;         /* This device can play */
  BOOLE channels[2];   /* Device supports Mono/Stereo flags */
  BOOLE bits[2];       /* Device supports 8/16 bits flags */
  ULO rates_max[2][2]; /* Maximum playback rate for [bits][chan] */
  soundDevModeSetFunc modeSetCallback;
  soundDevSampleAddFunc sampleAddCallback[2][2]; /* [BITS][CHANNELS] */
  soundDevEmulationStartingFunc emulationStartingCallback;
  soundDevEmulationStoppingFunc emulationStoppingCallback;
  soundDevBufferPlayFunc bufferPlayCallback;
} sound_device;

extern soundDevSampleAddFunc soundSampleAddCallback;


/*===========================*/
/* Symbols for configuration */
/*===========================*/

typedef enum {SOUND_8BITS, SOUND_16BITS} soundbits;
typedef enum {SOUND_MONO, SOUND_STEREO} soundchannels;
typedef enum {SOUND_15650, SOUND_22050, SOUND_31300, SOUND_44100} soundrates;
typedef enum {SOUND_NONE, SOUND_PLAY, SOUND_CONTIGUOUS, SOUND_EMULATE} soundmodes;
typedef enum {SOUND_FILTER_ORIGINAL, SOUND_FILTER_ALWAYS, SOUND_FILTER_NEVER} soundfilters;


/*=======*/
/* Other */
/*=======*/

typedef void (*soundStateFunc)(int);
typedef void (*soundFrequencyHandlerFunc)(void);

extern void soundstart(void); /* Code alignment check label */
extern ULO audiocounter;
extern ULO audio_hblanks;
extern void soundModeInit(void);
extern void soundStartup(void);


/*===============*/
/* Configuration */
/*===============*/

extern BOOLE sound_device_found;
extern soundmodes sound_mode;
extern soundrates sound_rate;
extern soundchannels sound_channels;
extern soundbits sound_bits;
extern BOOLE sound_wav_dump;
extern soundfilters sound_filter;
extern BOOLE sound_set_by_config;
extern ULO sound_buffer_depth;

extern ULO audstatejmptable[6];
extern WOR left[8],right[8];
extern ULO sound_framecounter;

extern ULO audpt[4];
extern ULO audlen[4];
extern ULO audper[4];
extern ULO audvol[4];
extern ULO auddat[4];

extern ULO audptw[4];
extern ULO audlenw[4];
extern ULO audpercounter[4];  
extern ULO auddatw[4];
extern soundStateFunc audstate[4];
extern ULO audvolw[4];

extern ULO periodtable[65536];
extern WOR volumes[256][64];

extern ULO audioirqmask[4];
extern ULO audiodmaconmask[4];

extern void audiostate0(int);
extern void audiostate1(int);
extern void audiostate2(int);
extern void audiostate3(int);
extern void audiostate4(int);
extern void audiostate5(int);

extern void audio_frequency_handler_44100(void);
extern void audio_frequency_handler_31300(void);
extern void audio_frequency_handler_22050(void);
extern void audio_frequency_handler_15650(void);


#endif


